
# 自定义的解压类型与key的mapping映射, key为unarchive目录下的脚本名称,key与下面UNARCHIVE_TYPE_COMMAND中的key保持一致
UNARCHIVE_TYPE_KEY_MAPPING = {
    "zip_process": [".zip"],
    "tar_bz_process": [".tar.bz", ".tar.bz2"],
    "unar_process": [".tar", ".tar.bz", ".tar.bz2"],
    "rar_process": [".rar"]
}


# 解压类型与解压命令、python处理脚本的module/class字典配置
UNARCHIVE_TYPE_COMMAND = {
    "zip_process": {
        "process_module": "zip_process",
        "process_class": "ZipProcess"
    },

    "tar_bz_process": {
        "process_module": "tar_bz_process",
        "process_class": "TarBzProcess"
    },

    "unar_process": {
        "process_module": "unar_process",
        "process_class": "UnarProcess"
    },

    "rar_process": {
        "process_module": "rar_process",
        "process_class": "RarProcess"
    }
}

UNARCHIVE_FILE_TYPES = [".7z", ".tar", ".tar.bz2", ".tar.gz", ".tar.lzma", ".tar.xz", ".rar", ".tgz", ".bz2", ".gz", ".lzma", ".xz", ".zip"]

# 日志打印出来的格式
LOG_LEVEL = 'INFO'
LOG_MODE = 'normal'
LOG_FORMAT = "{time:YYYY-MM-DD HH:mm:ss.SSS} | {level}    | {file}:{function}:{line} - {message}"
LOG_STDOUT_FORMAT = "<green>{time:YYYYMMDD HH:mm:ss}</green> | <level>{level}</level> | <cyan>{module}</cyan>.<cyan>{function}</cyan>:<cyan>{line}</cyan> - <level>{message}</level>"

__version__ = "0.0.1"



from allz.unarchive.tar_bz_process import TarBzProcess
from allz.unarchive.unar_process import UnarProcess
from allz.unarchive.zip_process import ZipProcess
from allz.unarchive.unarchive_main import Unarchive

__all__ = [
    "TarBzProcess",
    "UnarProcess",
    "ZipProcess",
    "Unarchive"
]



# -*- coding:utf-8 -*-
import sys

from allz.defs import LOG_LEVEL, LOG_STDOUT_FORMAT, LOG_MODE
from loguru import logger


def get_logger(name: str, log_mode=LOG_MODE):
    logger.remove()

    mylogger = logger.bind(name=name)
    if log_mode != LOG_MODE:
        logger.disable(name)
    else:
        logger.add(sys.stdout, format=LOG_STDOUT_FORMAT, level=LOG_LEVEL, filter=lambda record: record["extra"].get('name') == name)
        logger.enable(name)

    return mylogger


def on_success(src_path: str, des_path: str, log_mode=LOG_MODE) -> None:
    mylogger = get_logger("common", log_mode)
    mylogger.info(f"The compressed file {src_path} was successfully extracted to {des_path}")


def on_failure(src_path: str, des_path: str, log_mode=LOG_MODE) -> None:
    mylogger = get_logger("common", log_mode)
    mylogger.info(f"The compressed file {src_path} failed to be extracted to {des_path}")



"""Tests for the decompress module."""
# Standard library imports
import os
import pathlib
from pathlib import Path

from allz.unarchive import Unarchive
# from allz.unarchive.tar_bz_process import TarBzProcess

CURRENT_DIR = pathlib.Path(__file__).resolve().parent


# def test_tar_bz_process():
#     """test unarchive command"""
#     print(CURRENT_DIR)
#     src_path = Path.joinpath(CURRENT_DIR, "data/source/MNIST.tar.bz")
#     dest_path = Path.joinpath(CURRENT_DIR, "data/dest")
#     print(src_path, dest_path)

#     process = TarBzProcess()
#     process.main(src_path, dest_path)
#     assert Path.exists(dest_path) is True

def test_all_compress_type():
    dest_path_lst = []
    dest_file_name = '000000000001.jpg'
    archive_dir = Path.joinpath(CURRENT_DIR, "data/source")
    dest_dir = Path.joinpath(CURRENT_DIR, "data/dest") 
    for archive_file in os.listdir(archive_dir):
        is_file = os.path.isfile(Path.joinpath(archive_dir, archive_file))
        if is_file:
            src_path = "/".join([str(archive_dir), archive_file])
            dest_path = "/".join([str(dest_dir), archive_file + "#"])
            dest_path_lst.append(dest_path)
            Unarchive(src_path, dest_path)

    for dest_path in dest_path_lst:
        assert os.path.exists(dest_path) is True
        type_suffix = ".".join(dest_path.split(".")[1:])
        if type_suffix in ["jpg.bz2#", "jpg.gz#", "jpg.lzma#", "jpg.xz#"]:
            assert os.path.exists(dest_path + os.sep + dest_file_name) is True
        else:
            assert os.path.exists(dest_path + "/MNIST/media/" + dest_file_name) is True


if __name__ == '__main__':
    # test_tar_bz_process()
    test_all_compress_type()



from allz.libs.abstract_unarchive import AbstractUnarchive


class ZipProcess(AbstractUnarchive):
    def __init__(self):
        super().__init__()

    def handle(self, src_path, dest_path):
        cmd = f"7z x {src_path} -o{dest_path}".split()
        
        return cmd if cmd else None

    def decompress_test(self):
        cmd = "7z --help"
        
        return cmd if cmd else None



import os
import re

from allz.defs import UNARCHIVE_FILE_TYPES


class ArchiveTypeTester():
    """
    压缩文件判断器
    """
    def __init__(self):
        super().__init__()
        self.archive_file_type = UNARCHIVE_FILE_TYPES
        self._init_ext_regex()
        self.split_volume_archive = "\\d{2,3}$"

    def _init_ext_regex(self):
        if len(self.archive_file_type) == 0:
            raise Exception("archive_file_type is not configured")
        exts = []
        for ext in self.archive_file_type:
            exts.append(ext.strip(".\r\n").replace(".", "\\."))
        self.ext_regex = re.compile(".+\\.(" + "|".join(exts) + ")$")

    def is_archive(self, file_path):
        return self.ext_regex.match(file_path) is not None
    
    def is_split_volume_archive(self, file_path):
        matches = re.search(self.split_volume_archive, file_path)
        if matches:
            return True, matches.group()
        else:
            return False, None
    
    def _get_archive_type(self, file_path):
        path_splits = str(file_path).strip().split(".")
        suffix_last_two = "." + path_splits[-1]
        suffix_last_one = "." + ".".join(path_splits[-2: len(path_splits)])

        return suffix_last_two, suffix_last_one
    
    def is_support_archive_type(self, file_path):
        suffix_last_two, suffix_last_one = self._get_archive_type(file_path)
        if suffix_last_two in self.archive_file_type:
            return True, suffix_last_two
        elif suffix_last_one in self.archive_file_type:
            return True, suffix_last_one

        return False, None
    
    def get_split_volume_archives(self, file_path):
        is_archive, suffix_str = self.is_split_volume_archive(file_path)
        similar_archive_list = []
        if is_archive and suffix_str:
            prefix_path = str(file_path).strip("/").split("/")[-1][:-len(suffix_str)]
            current_dir = "/".join(str(file_path).split("/")[:-1])
            current_file_list = [ar_file for ar_file in os.listdir(current_dir) if os.path.isfile(current_dir + "/" + ar_file)]
            for file in current_file_list:
                if self.is_split_volume_archive(current_dir + "/" + file)[0] and (str(file)[:-len(suffix_str)] == prefix_path):
                    similar_archive_list.append(file)

        return similar_archive_list



import subprocess
import time
from abc import ABC, abstractmethod
from pathlib import Path

import allz.libs.common as common
from allz.defs import LOG_MODE


class AbstractUnarchive(ABC):
    def __init__(self):
        super().__init__()
        self.log = common.get_logger(self.__class__.__name__)

    @abstractmethod
    def handle(self, src_path, dest_path):
        pass

    def _handle(self, src_path, dest_path, log_mode):
        start_time = time.time()
        res_status = True
        stderr = ""

        if log_mode != LOG_MODE:
            self.log = common.get_logger(self.__class__.__name__, log_mode=log_mode)

        try:
            cmd = f"unar -q -D -o {dest_path} {src_path}".split()

            if not Path.exists(Path(dest_path)):
                Path(dest_path).mkdir(parents=True)

            handle_cmd = self.handle(src_path, dest_path)
            if handle_cmd:
                cmd = handle_cmd

            unar_res = subprocess.run(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            if unar_res.returncode != 0:
                self.log.exception(unar_res.stderr.decode('utf-8'))
                res_status = False
                stderr = unar_res.stderr.decode("utf-8")

            unar_res.check_returncode()
            self.log.info("The decompress command is: " + ' '.join(cmd))
            
        except Exception as e:
            elapsed = int((time.time() - start_time) * 1000) / 1000.0
            self.log.error(f"The compressed file {src_path} was processed with an error: {e}, elapsed time: {elapsed} 秒")
            self.log.exception(e)
            self.failed(src_path, dest_path, log_mode)
            res_status = False
            return res_status, stderr

        elapsed = int((time.time() - start_time) * 1000) / 1000.0
        self.log.info(f"The compressed file {src_path} was processed successfully, elapsed time: {elapsed} 秒")
        self.succeed(src_path, dest_path, log_mode)

        return res_status, stderr 

    @abstractmethod
    def decompress_test(self):
        pass
    
    def _decompress_test(self):
        decompress_cmd = self.decompress_test()
        if not decompress_cmd:
            return False

        try:
            decompress_res = subprocess.run(decompress_cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            if decompress_res.returncode != 0:
                self.log.exception(decompress_res.stderr.decode('utf-8'))

            decompress_res.check_returncode()
        except Exception as e:
            self.log.exception(e)
            return False
        
        return True

    def failed(self, src_path, dest_path, log_mode=LOG_MODE):
        common.on_failure(src_path, dest_path, log_mode)

    def succeed(self, src_path, dest_path, log_mode=LOG_MODE):
        common.on_success(src_path, dest_path, log_mode)

    def main(self, src_path, dest_path, log_mode=LOG_MODE):
        res_status, stderr = self._handle(src_path, dest_path, log_mode)
        return res_status, stderr



import setuptools

with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name="allz",
    version="0.0.3",
    author="opendatalab",
    author_email="yujia@pjlab.org.cn",
    description="A universal command line tool for compression and decompression",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/opendatalab/allz/",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'importlib',
    ],
    python_requires='>=3.7',
    keywords='python, compress, decompress, allz',
    entry_points={'console_scripts': [
        'allz = allz.cli.cmd:cli',
    ]},
)



# -*- coding: UTF-8 -*-

import sys

import click
import logging
from allz.defs import LOG_MODE, __version__
from allz.unarchive.unarchive_main import Unarchive, decompress_cmd_test

stderr_handler = logging.StreamHandler(stream=sys.stderr)


@click.group(context_settings={"help_option_names": ("-h", "--help"), "ignore_unknown_options": True})
@click.version_option(__version__)
def cli():
    pass


@cli.command("-d", help="To decompress file")
@click.option('--output-directory', '-o', default="./", help="The directory to write the contents of the archive. Defaults to the current directory.", required=False)
@click.option("-q", is_flag=True, required=False)
@click.argument('unkown_args', nargs=-1, type=click.UNPROCESSED)
def decompress(output_directory, unkown_args, q):
    src_path = ""
    log_mode = LOG_MODE

    if q:
        log_mode = "quiet"

    for arg in unkown_args:
        if not arg.startswith(("-", "--")):
            src_path = arg
            break

    res_status, stderr = Unarchive(src_path, output_directory, log_mode)
    sys.stderr.write(stderr)
    if not res_status:
        sys.exit(1)
    else:
        sys.exit(0)


@cli.command("check", help="Test which compressed files are supported")
def check_file_type():
    can, cannot = decompress_cmd_test()
    click.echo("The decompression types that are supported are: " + str(", ".join(can)))
    click.echo("The decompression types that are not supported are: " + str(", ".join(cannot)))


cli.add_command(decompress)
cli.add_command(check_file_type)


if __name__ == "__main__":
    cli()



import importlib

from allz.defs import (LOG_MODE, UNARCHIVE_TYPE_COMMAND, UNARCHIVE_TYPE_KEY_MAPPING)
from allz.libs.unarchive_tester import ArchiveTypeTester


def Unarchive(src_path, dest_path, log_mode=LOG_MODE):
    base_package_path = "allz.unarchive."

    # 1.判断压缩类型
    archiveTester = ArchiveTypeTester()
    is_archive = archiveTester.is_archive(src_path)
    if not is_archive:
        return False, "input compress file type test error"

    res, archive_type = archiveTester.is_support_archive_type(src_path)
    # 2.遍历配置的压缩类型找到对应的解压命令
    archive_type_cmd_init = "unar_process"
    for type_key, type_value in UNARCHIVE_TYPE_KEY_MAPPING.items():
        if archive_type in type_value:
            archive_type_cmd_init = type_key
            break
    
    archive_type_cmd_key = UNARCHIVE_TYPE_COMMAND[archive_type_cmd_init]
    process_module = archive_type_cmd_key['process_module']
    process_class = archive_type_cmd_key['process_class']

    # 3.动态调用解压脚本
    unar_module = importlib.import_module(base_package_path + process_module)
    unar_class = getattr(unar_module, process_class)
    unar_instance = unar_class()
    res_status, stderr = unar_instance.main(src_path, dest_path, log_mode)

    return res_status, str(stderr).strip()
        

def decompress_cmd_test():
    base_package_path = "allz.unarchive."
    can_process_type = []
    cannot_process_type = []

    for cmd_key, cmd_value in UNARCHIVE_TYPE_COMMAND.items():
        archive_type_cmd_key = UNARCHIVE_TYPE_COMMAND[cmd_key]
        process_module = archive_type_cmd_key['process_module']
        process_class = archive_type_cmd_key['process_class']

        unar_module = importlib.import_module(base_package_path + process_module)
        unar_class = getattr(unar_module, process_class)
        unar_instance = unar_class()
        res = unar_instance._decompress_test()
        
        if res:
            can_process_type.extend(UNARCHIVE_TYPE_KEY_MAPPING[cmd_key])
        else:
            cannot_process_type.extend(UNARCHIVE_TYPE_KEY_MAPPING[cmd_key])

    return list(set(can_process_type)), list(set(cannot_process_type))



from allz.libs.abstract_unarchive import AbstractUnarchive


class TarBzProcess(AbstractUnarchive):
    def __init__(self):
        super().__init__()

    def handle(self, src_path, dest_path):
        cmd = f"tar xjf {src_path} -C {dest_path}".split()

        return cmd if cmd else None

    def decompress_test(self):
        cmd = "tar --help"

        return cmd if cmd else None



from allz.libs.abstract_unarchive import AbstractUnarchive


class UnarProcess(AbstractUnarchive):
    def __init__(self):
        super().__init__()

    def handle(self, src_path, dest_path):
        cmd = f"unar -q -D -o {dest_path} {src_path}".split()

        return cmd if cmd else None
    
    def decompress_test(self):
        cmd = "unar --help"
        
        return cmd if cmd else None
